@@ -7,7 +7,7 @@ Attean::API::Query - Utility package defining query-related roles
 
 =head1 VERSION
 
-This document describes Attean::API::Query version 0.010
+This document describes Attean::API::Query version 0.013
 
 =head1 SYNOPSIS
 
@@ -24,7 +24,7 @@ This is a utility package for defining query-related roles:
 
 =cut
 
-package Attean::API::DirectedAcyclicGraph 0.010 {
+package Attean::API::DirectedAcyclicGraph 0.013 {
 	use Moo::Role;
 	use Scalar::Util qw(refaddr);
 	use Types::Standard qw(ArrayRef ConsumerOf);
@@ -134,118 +134,274 @@ package Attean::API::DirectedAcyclicGraph 0.010 {
 			$cb->( $self );
 		}
 	}
+
+	sub subpatterns_of_type {
+		my $self	= shift;
+		my @types	= @_;
+		my @p;
+		$self->walk( prefix => sub {
+			my $a	= shift;
+			foreach my $t (@types) {
+				push(@p, $a) if ($a->isa($t) or $a->does($t));
+			}
+		});
+		return @p;
+	}
 }
 
-package Attean::API::SPARQLSerializable 0.010 {
+package Attean::API::SPARQLSerializable 0.013 {
 	use AtteanX::SPARQL::Constants;
 	use AtteanX::SPARQL::Token;
 	use Moo::Role;
+	use Attean::API::Iterator;
+	use Attean::API::Serializer;
+	use AtteanX::Serializer::SPARQL;
 	use namespace::clean;
 
 	requires 'sparql_tokens';
 	
+	sub as_sparql {
+		my $self	= shift;
+		my $s		= AtteanX::Serializer::SPARQL->new();
+		my $i		= $self->sparql_tokens;
+		my $bytes	= $s->serialize_iter_to_bytes($i);
+		return $bytes;
+	}
+	
+	sub sparql_subtokens {
+		my $self	= shift;
+		if ($self->does('Attean::API::SPARQLQuerySerializable')) {
+			my $l	= AtteanX::SPARQL::Token->lbrace;
+			my $r	= AtteanX::SPARQL::Token->rbrace;
+			my @tokens;
+			push(@tokens, $l);
+			push(@tokens, $self->sparql_tokens->elements);
+			push(@tokens, $r);
+			return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+		} else {
+			return $self->sparql_tokens;
+		}
+	}
+	
+	sub dataset_tokens {
+		my $self	= shift;
+		my $dataset	= shift;
+		my @default	= @{ $dataset->{ default } || [] };
+		my @named	= @{ $dataset->{ named } || [] };
+		my $has_dataset	= (scalar(@default) + scalar(@named));
+		my @tokens;
+		if ($has_dataset) {
+			my $from		= AtteanX::SPARQL::Token->keyword('FROM');
+			my $named		= AtteanX::SPARQL::Token->keyword('NAMED');
+			foreach my $i (sort { $a->as_string cmp $b->as_string } @default) {
+				push(@tokens, $from);
+				push(@tokens, $i->sparql_tokens->elements);
+			}
+			foreach my $i (sort { $a->as_string cmp $b->as_string } @named) {
+				push(@tokens, $from);
+				push(@tokens, $named);
+				push(@tokens, $i->sparql_tokens->elements);
+			}
+		}
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
+	
 	sub query_tokens {
 		my $self	= shift;
+		my %args	= @_;
+		my $dataset	= $args{dataset} || {};
+		
+		my $as		= AtteanX::SPARQL::Token->keyword('AS');
+		my $lparen	= AtteanX::SPARQL::Token->lparen;
+		my $rparen	= AtteanX::SPARQL::Token->rparen;
 		
 		my $algebra	= $self;
 		
+		my %modifiers;
 		my $form	= 'SELECT';
 		if ($algebra->isa('Attean::Algebra::Ask')) {
 			$form	= 'ASK';
 			($algebra)	= @{ $algebra->children };
 		} elsif ($algebra->isa('Attean::Algebra::Describe')) {
 			$form	= 'DESCRIBE';
+			$modifiers{describe}	= $algebra->terms;
 			($algebra)	= @{ $algebra->children };
-			die;
 		} elsif ($algebra->isa('Attean::Algebra::Construct')) {
 			$form	= 'CONSTRUCT';
+			$modifiers{construct}	= $algebra->triples;
 			($algebra)	= @{ $algebra->children };
-			die;
 		}
 		
-		my %modifiers;
-		while ($algebra->isa('Attean::Algebra::Extend') or $algebra->isa('Attean::Algebra::Group') or $algebra->isa('Attean::Algebra::OrderBy') or $algebra->isa('Attean::Algebra::Distinct') or $algebra->isa('Attean::Algebra::Reduced') or $algebra->isa('Attean::Algebra::Slice') or $algebra->isa('Attean::Algebra::Project')) {
-			# TODO: Handle projection
-			# TODO: Handle aggregation/having
-			# TODO: Error if Slice appears before distinct/reduced
-			if ($algebra->isa('Attean::Algebra::Distinct')) {
-				$modifiers{ distinct }	= 1;
-			} elsif ($algebra->isa('Attean::Algebra::Reduced')) {
-				$modifiers{ reduced }	= 1;
-			} elsif ($algebra->isa('Attean::Algebra::Slice')) {
-				if ($algebra->limit >= 0) {
-					$modifiers{ limit }		= $algebra->limit;
+		unless ($form eq 'CONSTRUCT' or $form eq 'DESCRIBE') {
+			while ($algebra->isa('Attean::Algebra::Extend') or $algebra->isa('Attean::Algebra::Group') or $algebra->isa('Attean::Algebra::OrderBy') or $algebra->isa('Attean::Algebra::Distinct') or $algebra->isa('Attean::Algebra::Reduced') or $algebra->isa('Attean::Algebra::Slice') or $algebra->isa('Attean::Algebra::Project')) {
+				# TODO: Handle HAVING
+				# TODO: Error if Slice appears before distinct/reduced
+				if ($algebra->isa('Attean::Algebra::Distinct')) {
+					$modifiers{ distinct }	= 1;
+				} elsif ($algebra->isa('Attean::Algebra::Reduced')) {
+					$modifiers{ reduced }	= 1;
+				} elsif ($algebra->isa('Attean::Algebra::Slice')) {
+					if ($algebra->limit >= 0) {
+						$modifiers{ limit }		= $algebra->limit;
+					}
+					if ($algebra->offset > 0) {
+						$modifiers{ offset }	= $algebra->offset;
+					}
+				} elsif ($algebra->isa('Attean::Algebra::OrderBy')) {
+					$modifiers{order}	= $algebra->comparators;
+				} elsif ($algebra->isa('Attean::Algebra::Extend')) {
+					my $v		= $algebra->variable;
+					my $name	= $v->value;
+					my $expr	= $algebra->expression;
+					my @tokens;
+					push(@tokens, $lparen);
+					push(@tokens, $expr->sparql_tokens->elements);
+					push(@tokens, $as);
+					push(@tokens, $v->sparql_tokens->elements);
+					push(@tokens, $rparen);
+					$modifiers{project_expression_tokens}{$name}	= \@tokens;
+				} elsif ($algebra->isa('Attean::Algebra::Project')) {
+					my $vars	= $algebra->variables;
+					my ($child)	= @{ $algebra->children };
+					my @vars	= sort(map { $_->value } @$vars);
+					my @subvars	= sort($child->in_scope_variables);
+					if (scalar(@vars) == scalar(@subvars) and join('.', @vars) eq join('.', @subvars)) {
+						# this is a SELECT * query
+					} else {
+						foreach my $v (@$vars) {
+							my $name	= $v->value;
+							unless ($modifiers{project_variables}{$name}++) {
+								push(@{ $modifiers{project_variables_order} }, $name);
+							}
+						}
+					}
+				} elsif ($algebra->isa('Attean::Algebra::Group')) {
+					my $aggs	= $algebra->aggregates;
+					my $groups	= $algebra->groupby;
+					foreach my $agg (@$aggs) {
+						my $v		= $agg->variable;
+						my $name	= $v->value;
+						my @tokens;
+						push(@tokens, $lparen);
+						push(@tokens, $agg->sparql_tokens->elements);
+						push(@tokens, $as);
+						push(@tokens, $v->sparql_tokens->elements);
+						push(@tokens, $rparen);
+						unless ($modifiers{project_variables}{$name}++) {
+							push(@{ $modifiers{project_variables_order} }, $name);
+						}
+						$modifiers{project_expression_tokens}{$name}	= \@tokens;
+					}
+					foreach my $group (@$groups) {
+						push(@{ $modifiers{groups} }, $group->sparql_tokens->elements);
+					}
+				} else {
+					die "Unexpected pattern type encountered in query_tokens: " . ref($algebra);
 				}
-				if ($algebra->offset > 0) {
-					$modifiers{ offset }	= $algebra->offset;
-				}
-			} elsif ($algebra->isa('Attean::Algebra::OrderBy')) {
-				$modifiers{order}	= $algebra->comparators;
-			} elsif ($algebra->isa('Attean::Algebra::Extend')) {
-				die;
-				# $modifiers{project}	= [];
-			} elsif ($algebra->isa('Attean::Algebra::Project')) {
-				die;
-				# $modifiers{project}	= [];
-			} elsif ($algebra->isa('Attean::Algebra::Group')) {
-				die;
-				# $modifiers{having}	= $expr;
-			} else {
-				die;
+				($algebra)	= @{ $algebra->children };
 			}
-			($algebra)	= @{ $algebra->children };
 		}
 		
 		my @tokens;
+
+		my $where	= AtteanX::SPARQL::Token->keyword('WHERE');
+		my $lbrace	= AtteanX::SPARQL::Token->lbrace;
+		my $rbrace	= AtteanX::SPARQL::Token->rbrace;
+
+
 		if ($form eq 'SELECT') {
-			push(@tokens, AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['SELECT'] ));
+			push(@tokens, AtteanX::SPARQL::Token->keyword('SELECT'));
 			if ($modifiers{distinct}) {
-				push(@tokens, AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['DISTINCT'] ));
+				push(@tokens, AtteanX::SPARQL::Token->keyword('DISTINCT'));
 			} elsif ($modifiers{reduced}) {
-				push(@tokens, AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['REDUCED'] ));
+				push(@tokens, AtteanX::SPARQL::Token->keyword('REDUCED'));
 			}
 			
-			if (my $p = $modifiers{project}) {
-				die;
+			if (my $p = $modifiers{project_variables_order}) {
+				foreach my $name (@$p) {
+					if (my $etokens = $modifiers{project_expression_tokens}{$name}) {
+						push(@tokens, @$etokens);
+					} else {
+						my $v	= Attean::Variable->new( value => $name );
+						push(@tokens, $v->sparql_tokens->elements);
+					}
+				}
 			} else {
-				push(@tokens, AtteanX::SPARQL::Token->fast_constructor( STAR, -1, -1, -1, -1, ['*'] ));
+				push(@tokens, AtteanX::SPARQL::Token->star);
+			}
+			
+			push(@tokens, $self->dataset_tokens($dataset)->elements);
+			
+			push(@tokens, $where);
+			if ($algebra->isa('Attean::Algebra::Join')) {
+				# don't emit extraneous braces at the top-level
+				push(@tokens, $algebra->sparql_tokens->elements);
+			} else {
+				push(@tokens, $lbrace);
+				push(@tokens, $algebra->sparql_tokens->elements);
+				push(@tokens, $rbrace);
+			}
+			if (my $groups = $modifiers{groups}) {
+				push(@tokens, AtteanX::SPARQL::Token->keyword('GROUP'));
+				push(@tokens, AtteanX::SPARQL::Token->keyword('BY'));
+				push(@tokens, @$groups);
 			}
-			push(@tokens, AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['WHERE'] ));
-			push(@tokens, AtteanX::SPARQL::Token->fast_constructor( LBRACE, -1, -1, -1, -1, ['{'] ));
-			push(@tokens, $algebra->sparql_tokens->elements);
-			push(@tokens, AtteanX::SPARQL::Token->fast_constructor( RBRACE, -1, -1, -1, -1, ['}'] ));
 			if (my $expr = $modifiers{having}) {
-				push(@tokens, AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['HAVING'] ));
+				push(@tokens, AtteanX::SPARQL::Token->keyword('HAVING'));
 				push(@tokens, $expr->sparql_tokens->elements);
 			}
 			if (my $comps = $modifiers{order}) {
-				push(@tokens, AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['ORDER'] ));
-				push(@tokens, AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['BY'] ));
+				push(@tokens, AtteanX::SPARQL::Token->keyword('ORDER'));
+				push(@tokens, AtteanX::SPARQL::Token->keyword('BY'));
 				foreach my $c (@$comps) {
 					push(@tokens, $c->sparql_tokens->elements);
 				}
 			}
 			if (exists $modifiers{limit}) {
-				push(@tokens, AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['LIMIT'] ));
-				push(@tokens, AtteanX::SPARQL::Token->fast_constructor( INTEGER, -1, -1, -1, -1, [$modifiers{limit}] ));
+				push(@tokens, AtteanX::SPARQL::Token->keyword('LIMIT'));
+				push(@tokens, AtteanX::SPARQL::Token->integer($modifiers{limit}));
 			}
 			if (exists $modifiers{offset}) {
-				push(@tokens, AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['OFFSET'] ));
-				push(@tokens, AtteanX::SPARQL::Token->fast_constructor( INTEGER, -1, -1, -1, -1, [$modifiers{offset}] ));
+				push(@tokens, AtteanX::SPARQL::Token->keyword('OFFSET'));
+				push(@tokens, AtteanX::SPARQL::Token->integer($modifiers{offset}));
 			}
-		} elsif ($form eq 'ASK') {
-			push(@tokens, AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['SELECT'] ));
-			push(@tokens, AtteanX::SPARQL::Token->fast_constructor( LBRACE, -1, -1, -1, -1, ['{'] ));
+		} elsif ($form eq 'DESCRIBE') {
+			push(@tokens, AtteanX::SPARQL::Token->keyword('DESCRIBE'));
+			foreach my $t (@{ $modifiers{describe} }) {
+				push(@tokens, $t->sparql_tokens->elements);
+			}
+			push(@tokens, $self->dataset_tokens($dataset)->elements);
+			push(@tokens, $where);
+			push(@tokens, $lbrace);
 			push(@tokens, $algebra->sparql_tokens->elements);
-			push(@tokens, AtteanX::SPARQL::Token->fast_constructor( RBRACE, -1, -1, -1, -1, ['}'] ));
+			push(@tokens, $rbrace);
+		} elsif ($form eq 'CONSTRUCT') {
+			push(@tokens, AtteanX::SPARQL::Token->keyword('CONSTRUCT'));
+			push(@tokens, $lbrace);
+			foreach my $t (@{ $modifiers{construct} }) {
+				push(@tokens, $t->sparql_tokens->elements);
+				push(@tokens, AtteanX::SPARQL::Token->dot);
+			}
+			push(@tokens, $rbrace);
+			push(@tokens, $self->dataset_tokens($dataset)->elements);
+			push(@tokens, $where);
+			push(@tokens, $lbrace);
+			push(@tokens, $algebra->sparql_tokens->elements);
+			push(@tokens, $rbrace);
+		} elsif ($form eq 'ASK') {
+			push(@tokens, AtteanX::SPARQL::Token->keyword('ASK'));
+			push(@tokens, $self->dataset_tokens($dataset)->elements);
+			push(@tokens, $lbrace);
+			push(@tokens, $algebra->sparql_tokens->elements);
+			push(@tokens, $rbrace);
 		} else {
-			die;
+			die "Unexpected query for '$form' in query_tokens";
 		}
 		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
 	}
 }
 
-package Attean::API::SPARQLQuerySerializable 0.010 {
+package Attean::API::SPARQLQuerySerializable 0.013 {
 	use Moo::Role;
 	use namespace::clean;
 	with 'Attean::API::SPARQLSerializable';
@@ -260,9 +416,11 @@ package Attean::API::SPARQLQuerySerializable 0.010 {
 
 =cut
 
-package Attean::API::Algebra 0.010 {
+package Attean::API::Algebra 0.013 {
 	use Moo::Role;
 
+	with 'Attean::API::SPARQLSerializable';
+	
 	requires 'as_sparql';
 	requires 'in_scope_variables';			# variables that will be in-scope after this operation is evaluated
 	
@@ -285,24 +443,11 @@ package Attean::API::Algebra 0.010 {
 			my $level	= shift;
 			my $parent	= shift;
 			my $indent	= '  ' x $level;
-			$string	.= "-$indent " .  $a->algebra_as_string . "\n";
+			$string	.= "-$indent " .  $a->algebra_as_string($level) . "\n";
 		});
 		return $string;
 	}
 	
-	sub subpatterns_of_type {
-		my $self	= shift;
-		my @types	= @_;
-		my @p;
-		$self->walk( prefix => sub {
-			my $a	= shift;
-			foreach my $t (@types) {
-				push(@p, $a) if ($a->isa($t) or $a->does($t));
-			}
-		});
-		return @p;
-	}
-	
 	sub blank_nodes {
 		my $self	= shift;
 		my %blanks;
@@ -348,7 +493,7 @@ package Attean::API::Algebra 0.010 {
 
 =cut
 
-package Attean::API::QueryTree 0.010 {
+package Attean::API::QueryTree 0.013 {
 	use Moo::Role;
 	with 'Attean::API::DirectedAcyclicGraph';
 }
@@ -357,7 +502,7 @@ package Attean::API::QueryTree 0.010 {
 
 =cut
 
-package Attean::API::NullaryQueryTree 0.010 {
+package Attean::API::NullaryQueryTree 0.013 {
 	use Moo::Role;
 	sub arity { return 0 }
 	with 'Attean::API::QueryTree';
@@ -367,17 +512,22 @@ package Attean::API::NullaryQueryTree 0.010 {
 
 =cut
 
-package Attean::API::UnaryQueryTree 0.010 {
+package Attean::API::UnaryQueryTree 0.013 {
 	use Moo::Role;
 	sub arity { return 1 }
 	with 'Attean::API::QueryTree';
+	
+	sub child {
+		my $self	= shift;
+		return $self->children->[0];
+	}
 }
 
 =item * L<Attean::API::BinaryQueryTree>
 
 =cut
 
-package Attean::API::BinaryQueryTree 0.010 {
+package Attean::API::BinaryQueryTree 0.013 {
 	use Moo::Role;
 	sub arity { return 2 }
 	with 'Attean::API::QueryTree';
@@ -387,7 +537,7 @@ package Attean::API::BinaryQueryTree 0.010 {
 
 =cut
 
-package Attean::API::PropertyPath 0.010 {
+package Attean::API::PropertyPath 0.013 {
 	use Moo::Role;
 	with 'Attean::API::QueryTree';
 	requires 'as_string';
@@ -398,7 +548,7 @@ package Attean::API::PropertyPath 0.010 {
 
 =cut
 
-package Attean::API::UnaryPropertyPath 0.010 {
+package Attean::API::UnaryPropertyPath 0.013 {
 	use Moo::Role;
 	use Types::Standard qw(ConsumerOf);
 	use namespace::clean;
@@ -428,7 +578,7 @@ package Attean::API::UnaryPropertyPath 0.010 {
 
 =cut
 
-package Attean::API::NaryPropertyPath 0.010 {
+package Attean::API::NaryPropertyPath 0.013 {
 	use Moo::Role;
 	use Types::Standard qw(ArrayRef ConsumerOf);
 	use namespace::clean;
@@ -455,7 +605,7 @@ package Attean::API::NaryPropertyPath 0.010 {
 
 =cut
 
-package Attean::API::UnionScopeVariables 0.010 {
+package Attean::API::UnionScopeVariables 0.013 {
 	use Moo::Role;
 	sub in_scope_variables {
 		my $self	= shift;
@@ -471,7 +621,7 @@ package Attean::API::UnionScopeVariables 0.010 {
 
 =cut
 
-package Attean::API::IntersectionScopeVariables 0.010 {
+package Attean::API::IntersectionScopeVariables 0.013 {
 	use Moo::Role;
 	sub in_scope_variables {
 		my $self	= shift;
@@ -505,7 +655,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014 Gregory Todd Williams.
+Copyright (c) 2014--2016 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
