@@ -7,7 +7,7 @@ Attean::API::Serializer - Serializer role
 
 =head1 VERSION
 
-This document describes Attean::API::Serializer version 0.011
+This document describes Attean::API::Serializer version 0.013
 
 =head1 DESCRIPTION
 
@@ -68,7 +68,7 @@ UTF-8 encoded byte string.
 
 use Type::Tiny;
 
-package Attean::API::Serializer 0.011 {
+package Attean::API::Serializer 0.013 {
 	use Moo::Role;
 	
 	requires 'canonical_media_type'; # => (is => 'ro', isa => 'Str', init_arg => undef);
@@ -92,7 +92,7 @@ package Attean::API::Serializer 0.011 {
 	}
 }
 
-package Attean::API::AbbreviatingSerializer 0.011 {
+package Attean::API::AbbreviatingSerializer 0.013 {
 	# Serializer that can make use of a base IRI and/or prefix IRI mappings
 	use Moo::Role;
 	use Types::Standard qw(InstanceOf ConsumerOf Maybe);
@@ -104,13 +104,13 @@ package Attean::API::AbbreviatingSerializer 0.011 {
 	has namespaces	=> (is => 'ro', isa => Maybe[InstanceOf['URI::NamespaceMap']], predicate => 'has_namespaces');
 }
 
-package Attean::API::AppendableSerializer 0.011 {
+package Attean::API::AppendableSerializer 0.013 {
 	# Serializer for a format that allows multiple serialization calls to be appended and remain syntactically valid
 	use Moo::Role;
 	with 'Attean::API::Serializer';
 }
 
-package Attean::API::TermSerializer 0.011 {
+package Attean::API::TermSerializer 0.013 {
 	use Moo::Role;
 	with 'Attean::API::Serializer';
 	sub handled_type {
@@ -119,7 +119,7 @@ package Attean::API::TermSerializer 0.011 {
 	}
 }
 
-package Attean::API::TripleSerializer 0.011 {
+package Attean::API::TripleSerializer 0.013 {
 	use Moo::Role;
 	with 'Attean::API::Serializer';
 	sub handled_type {
@@ -128,7 +128,7 @@ package Attean::API::TripleSerializer 0.011 {
 	}
 }
 
-package Attean::API::QuadSerializer 0.011 {
+package Attean::API::QuadSerializer 0.013 {
 	use Moo::Role;
 	with 'Attean::API::Serializer';
 	
@@ -138,7 +138,7 @@ package Attean::API::QuadSerializer 0.011 {
 	}
 }
 
-package Attean::API::MixedStatementSerializer 0.011 {
+package Attean::API::MixedStatementSerializer 0.013 {
 	use Moo::Role;
 	with 'Attean::API::Serializer';
 	
@@ -148,13 +148,36 @@ package Attean::API::MixedStatementSerializer 0.011 {
 	}
 }
 
-package Attean::API::ResultSerializer 0.011 {
+package Attean::API::ResultSerializer 0.013 {
 	use Moo::Role;
 	with 'Attean::API::Serializer';
 	sub handled_type {
 		state $ITEM_TYPE = Type::Tiny::Role->new(role => 'Attean::API::Result');
 		return $ITEM_TYPE;
 	}
+	
+	around 'serialize_list_to_io' => sub {
+		my $orig	= shift;
+		my $self	= shift;
+		my $io		= shift;
+		my @vars;
+		if (scalar(@_)) {
+			@vars	= $_[0]->variables;
+		}
+		my $iter	= Attean::ListIterator->new( values => [@_], item_type => $self->handled_type->role, variables => \@vars );
+		return $self->serialize_list_to_io($io, $iter);
+	};
+	
+	around 'serialize_list_to_bytes' => sub {
+		my $orig	= shift;
+		my $self	= shift;
+		my @vars;
+		if (scalar(@_)) {
+			@vars	= $_[0]->variables;
+		}
+		my $iter	= Attean::ListIterator->new( values => [@_], item_type => $self->handled_type->role, variables => \@vars );
+		return $self->serialize_iter_to_bytes($iter);
+	};
 }
 
 1;
