@@ -7,7 +7,7 @@ Attean::Literal - RDF Literals
 
 =head1 VERSION
 
-This document describes Attean::Literal version 0.010
+This document describes Attean::Literal version 0.013
 
 =head1 SYNOPSIS
 
@@ -24,9 +24,31 @@ This document describes Attean::Literal version 0.010
 The Attean::Literal class represents RDF literals.
 It conforms to the L<Attean::API::Literal|Attean::API::Term> role.
 
+=head1 ATTRIBUTES
+
+The following attributes exist:
+
+=over 4
+
+=item C<< value >>
+
+=item C<< language >>
+
+=item C<< datatype >>
+
+=back
+
+=head1 METHODS
+
+=over 4
+
+=item C<< has_language >>
+
+Returns true if the literal has a language tag, false otherwise.
+
 =cut
 
-package Attean::Literal 0.010 {
+package Attean::Literal 0.013 {
 	use Moo;
 	use Types::Standard qw(Str Maybe InstanceOf);
 	use Attean::API::Term;
@@ -70,7 +92,7 @@ package Attean::Literal 0.010 {
 		my $orig	= shift;
 		my $self	= shift;
 		if ($self->has_language) {
-			return Attean::IRI->new(value => 'http://www.w3.org/2001/XMLSchema#string');
+			return Attean::IRI->new(value => 'http://www.w3.org/2001/XMLSchema#langString');
 		} else {
 			return $self->$orig(@_);
 		}
@@ -95,11 +117,23 @@ package Attean::Literal 0.010 {
 		}
 	}
 
+=item C<< true >>
+
+The xsd:true term.
+
+=cut
+
 	sub true {
 		state $v	= Attean::Literal->new( value => 'true', datatype => 'http://www.w3.org/2001/XMLSchema#boolean' );
 		return $v;
 	}
 	
+=item C<< false >>
+
+The xsd:false term.
+
+=cut
+
 	sub false {
 		state $v	= Attean::Literal->new( value => 'false', datatype => 'http://www.w3.org/2001/XMLSchema#boolean' );
 		return $v;
@@ -124,6 +158,8 @@ package Attean::Literal 0.010 {
 
 __END__
 
+=back
+
 =head1 BUGS
 
 Please report any bugs or feature requests to through the GitHub web interface
@@ -139,7 +175,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014 Gregory Todd Williams.
+Copyright (c) 2014--2016 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
