@@ -9,7 +9,7 @@ AtteanX::API::JoinRotatingPlanner - Query planning role to produce alternative j
 
 =head1 VERSION
 
-This document describes AtteanX::API::JoinRotatingPlanner version 0.026
+This document describes AtteanX::API::JoinRotatingPlanner version 0.013
 
 =head1 DESCRIPTION
 
@@ -35,13 +35,13 @@ plan.
 
 =cut
 
-package AtteanX::API::JoinRotatingPlanner 0.026 {
+package AtteanX::API::JoinRotatingPlanner {
 	# Rotate joins like (A⋈B)⋈C to A⋈(B⋈C), with the ability to coalesce B⋈C (e.g. for adjacent BGPs)
+	use Moo::Role;
 	use Attean;
 	use Attean::RDF;
-
-	use Moo::Role;
-
+	use namespace::clean;
+	
 	requires 'coalesce_rotated_join';
 	requires 'allow_join_rotation';
 	
@@ -108,7 +108,7 @@ at L<https://github.com/kasei/attean/issues>.
 
 =head1 SEE ALSO
 
-
+L<http://www.perlrdf.org/>
 
 =head1 AUTHOR
 
@@ -116,7 +116,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2020 Gregory Todd Williams.
+Copyright (c) 2014--2016 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
