@@ -10,7 +10,7 @@ AtteanX::Parser::Turtle - Turtle RDF Parser
 
 =head1 VERSION
 
-This document describes AtteanX::Parser::Turtle version 0.010
+This document describes AtteanX::Parser::Turtle version 0.013
 
 =head1 SYNOPSIS
 
@@ -22,13 +22,29 @@ This document describes AtteanX::Parser::Turtle version 0.010
 
 This module implements a parser for the Turtle RDF format.
 
+=head1 ATTRIBUTES
+
+=over 4
+
+=item C<< canonical_media_type >>
+
+=item C<< media_types >>
+
+=item C<< file_extensions >>
+
+=item C<< canonicalize >>
+
+A boolean indicating whether term values should be canonicalized during parsing.
+
+=back
+
 =head1 METHODS
 
 =over 4
 
 =cut
 
-package AtteanX::Parser::Turtle 0.010 {
+package AtteanX::Parser::Turtle 0.013 {
 	use Moo;
 	use Types::Standard qw(Bool ArrayRef HashRef Str Maybe InstanceOf);
 	use utf8;
@@ -40,37 +56,25 @@ package AtteanX::Parser::Turtle 0.010 {
 	use Attean::API::Parser;
 	use namespace::clean;
 
-=item C<< canonical_media_type >>
-
-Returns the canonical media type for Turtle: text/turtle.
-
-=cut
-
 	sub canonical_media_type { return "text/turtle" }
 
-=item C<< media_types >>
-
-Returns a list of media types that may be parsed with the Turtle parser:
-text/turtle.
-
-=cut
-
 	sub media_types {
 		return [qw(application/x-turtle application/turtle text/turtle)];
 	}
 
-=item C<< file_extensions >>
-
-Returns a list of file extensions that may be parsed with the parser.
-
-=cut
-
 	sub file_extensions { return [qw(ttl)] }
 
 	has 'canonicalize'	=> (is => 'rw', isa => Bool, default => 0);
-	has 'map' => (is => 'ro', isa => HashRef[Str], default => sub { +{} });
+	has '_map' => (is => 'ro', isa => HashRef[Str], default => sub { +{} });
+
+=item C<< has_namespaces >>
+
+Returns true if the parser has a namespace map, false otherwise.
+
+=cut
+
 	has 'namespaces' => (is => 'rw', isa => Maybe[InstanceOf['URI::NamespaceMap']], predicate => 'has_namespaces');
-	has	'stack'	=> (
+	has	'_stack'	=> (
 		is => 'ro',
 		isa => ArrayRef,
 		default => sub { [] },
@@ -123,15 +127,26 @@ the data read from the UTF-8 encoded byte string C<< $data >>.
 		$self->_parse($l);
 	}
 
-=item C<< parse_node ( $string, $base ) >>
+=item C<< parse_term_from_string( $bytes ) >>
+
+This method is deprecated, as the name was misleading.
+Calls should be replaced with C<< parse_term_from_bytes >>.
+	
+=item C<< parse_term_from_bytes ( $bytes, $base ) >>
+
+=item C<< parse_node ( $bytes, $base ) >>
 
 Returns the Attean::API::Term object corresponding to the node whose N-Triples
-serialization is found at the beginning of C<< $string >>.
+serialization is found at the beginning of C<< $bytes >>.
 
 =cut
 
-	# TODO: this method is badly named. it actually accepts bytes, not strings.
 	sub parse_term_from_string {
+# 		TODO: warn "parse_term_from_string is now called parse_term_from_bytes\n";
+		return shift->parse_term_from_bytes(@_);
+	}
+	
+	sub parse_term_from_bytes {
 		my $self	= shift;
 		return $self->parse_node(@_);
 	}
@@ -162,14 +177,14 @@ serialization is found at the beginning of C<< $string >>.
 	sub _unget_token {
 		my $self	= shift;
 		my $t		= shift;
-		push(@{ $self->stack }, $t);
+		push(@{ $self->_stack }, $t);
 # 		push(@{ $self->{ stack } }, $t);
 	}
 
 	sub _next_nonws {
 		my $self	= shift;
-		if (scalar(@{ $self->stack })) {
-			return pop(@{ $self->stack });
+		if (scalar(@{ $self->_stack })) {
+			return pop(@{ $self->_stack });
 		}
 		my $l		= shift;
 		while (1) {
@@ -203,7 +218,7 @@ serialization is found at the beginning of C<< $string >>.
 		my $t		= shift;
 		my $type	= $t->type;
 	# 		when (WS) {}
-		if ($type == PREFIX or $type == SPARQLPREFIX) {
+		if ($type == TURTLEPREFIX or $type == PREFIX) {
 			$t	= $self->_get_token_type($l, PREFIXNAME);
 			use Data::Dumper;
 			unless (defined($t->value)) {
@@ -220,14 +235,14 @@ serialization is found at the beginning of C<< $string >>.
 			}
 			my $r	= Attean::IRI->new(%args);
 			my $iri	= $r->as_string;
-			if ($type == PREFIX) {
+			if ($type == TURTLEPREFIX) {
 				$t	= $self->_get_token_type($l, DOT);
 	# 			$t	= $self->_next_nonws($l);
 	# 			if ($t and $t->type != DOT) {
 	# 				$self->_unget_token($t);
 	# 			}
 			}
-			$self->map->{$name}	= $iri;
+			$self->_map->{$name}	= $iri;
 			if ($self->has_namespaces) {
 				my $ns	= $self->namespaces;
 				unless ($ns->namespace_uri($name)) {
@@ -235,7 +250,7 @@ serialization is found at the beginning of C<< $string >>.
 				}
 			}
 		}
-		elsif ($type == BASE or $type == SPARQLBASE) {
+		elsif ($type == TURTLEBASE or $type == BASE) {
 			$t	= $self->_get_token_type($l, IRI);
 			my %args	= (value => $t->value);
 			if ($self->has_base) {
@@ -243,7 +258,7 @@ serialization is found at the beginning of C<< $string >>.
 			}
 			my $r	= Attean::IRI->new(%args);
 			my $iri	= $r->as_string;
-			if ($type == BASE) {
+			if ($type == TURTLEBASE) {
 				$t	= $self->_get_token_type($l, DOT);
 	# 			$t	= $self->_next_nonws($l);
 	# 			if ($t and $t->type != DOT) {
@@ -515,10 +530,10 @@ serialization is found at the beginning of C<< $string >>.
 		elsif ($type eq PREFIXNAME) {
 			my ($ns, $local)	= @{ $t->args };
 			$ns		=~ s/:$//;
-			unless (exists $self->map->{$ns}) {
+			unless (exists $self->_map->{$ns}) {
 				$self->_throw_error("Use of undeclared prefix '$ns'", $t);
 			}
-			my $prefix			= $self->map->{$ns};
+			my $prefix			= $self->_map->{$ns};
 			no warnings 'uninitialized';
 			my $iri				= Attean::IRI->new("${prefix}${local}");
 			return $iri;
@@ -570,7 +585,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2006-2012 Gregory Todd Williams. This
+Copyright (c) 2014--2016 Gregory Todd Williams. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
