@@ -7,14 +7,12 @@ AtteanX::SPARQL::Token - Token objects used for parsing and serializing SPARQL
 
 =head1 VERSION
 
-This document describes AtteanX::SPARQL::Token version 0.011
+This document describes AtteanX::SPARQL::Token version 0.013
 
 =head1 SYNOPSIS
 
   use v5.14;
   use Attean;
-  my $term = Attean::Blank->new('b1');
-  $term->ntriples_string; # _:b1
 
 =head1 DESCRIPTION
 
@@ -57,7 +55,7 @@ An array of values associated with the token (e.g. the integer value of an INT t
 
 =cut
 
-package AtteanX::SPARQL::Token;
+package AtteanX::SPARQL::Token 0.013;
 
 use Moo;
 use Types::Standard qw(ArrayRef Str);
@@ -73,6 +71,8 @@ has line => ( is => 'ro', );
 has column => ( is => 'ro', );
 has args => ( is => 'ro', isa => ArrayRef[Str]);
 
+extends 'AtteanX::Parser::Turtle::Token';
+
 =item C<< value >>
 
 Returns the token value.
@@ -160,6 +160,37 @@ sub keyword {
 	return $class->fast_constructor(KEYWORD, -1, -1, -1, -1, [uc($kw)]);
 }
 
+=item C<< integer( $value ) >>
+
+Returns a new L<AtteanX::SPARQL::Token> object with the C<INTEGER> type and
+the given C<$value>.
+
+=cut
+
+sub integer {
+	my $class	= shift;
+	my $value	= shift;
+	return $class->fast_constructor(INTEGER, -1, -1, -1, -1, [+$value] );
+}
+
+=item C<< as_string >>
+
+Returns a string description of the token including the token type and any
+associated values.
+
+=cut
+
+sub as_string {
+	my $self	= shift;
+	my $type	= decrypt_constant($self->type);
+	my @args	= @{ $self->args };
+	if (scalar(@args)) {
+		return "$type(" . join(', ', @args) . ")";
+	} else {
+		return $type;
+	}
+}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
