@@ -4,7 +4,7 @@ AtteanX::Serializer::SPARQLXML - SPARQL Results XML Serializer
 
 =head1 VERSION
 
-This document describes AtteanX::Serializer::SPARQLXML version 0.011
+This document describes AtteanX::Serializer::SPARQLXML version 0.013
 
 =head1 SYNOPSIS
 
@@ -33,10 +33,10 @@ This document describes AtteanX::Serializer::SPARQLXML version 0.011
 use v5.14;
 use warnings;
 
-package AtteanX::Serializer::SPARQLXML 0.011 {
+package AtteanX::Serializer::SPARQLXML 0.013 {
 	use Moo;
 	use Types::Standard qw(Str ArrayRef);
-	use Encode qw(encode);
+	use Encode qw(encode encode_utf8);
 	use Scalar::Util qw(blessed);
 	use Attean::ListIterator;
 	use List::MoreUtils qw(any);
@@ -70,8 +70,9 @@ L<IO::Handle> object C<< $fh >>.
 <sparql xmlns="http://www.w3.org/2005/sparql-results#">
 <head>
 END
+		my @vars	= @{ $iter->variables };
 		if ($iter->does('Attean::API::ResultIterator')) {
-			foreach my $v (@{ $iter->variables }) {
+			foreach my $v (@vars) {
 				print $fh qq(\t<variable name="$v"/>\n);
 			}
 		}
@@ -83,7 +84,7 @@ END
 		
 		while (my $t = $iter->next()) {
 			print $fh "\t\t<result>\n";
-			foreach my $name ($t->variables) {
+			foreach my $name (@vars) {
 				my $term	= $t->value($name);
 				if (blessed($term)) {
 					if ($term->does('Attean::API::IRI')) {
@@ -91,12 +92,14 @@ END
 						$label	=~ s/&/&amp;/g;
 						$label	=~ s/</&lt;/g;
 						$label	=~ s/"/&quot;/g;
+						$label	= encode_utf8($label);
 						print $fh qq(\t\t\t<binding name="${name}"><uri>${label}</uri></binding>\n);
 					} elsif ($term->does('Attean::API::Literal')) {
 						my $label	= $term->value;
 						$label	=~ s/&/&amp;/g;
 						$label	=~ s/</&lt;/g;
 						$label	=~ s/"/&quot;/g;
+						$label	= encode_utf8($label);
 						if (my $lang = $term->language) {
 							$label	= qq(<literal xml:lang="${lang}">${label}</literal>);
 						} elsif (my $dt = $term->datatype) {
@@ -110,6 +113,7 @@ END
 						$label	=~ s/&/&amp;/g;
 						$label	=~ s/</&lt;/g;
 						$label	=~ s/"/&quot;/g;
+						$label	= encode_utf8($label);
 						print $fh qq(\t\t\t<binding name="${name}"><bnode>${label}</bnode></binding>\n);
 					} else {
 						die "Term object has an unrecognized type: " . ref($term);
